package com.usky.oa.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.*;
import com.usky.oa.mapper.OaFlowMapper;
import com.usky.oa.mapper.OaNodeMapper;
import com.usky.oa.mapper.SysDeptMapper;
import com.usky.oa.mapper.SysUserMapper;
import com.usky.system.domain.SysUser;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @author fyc
 * @email yuchuan.fu@chinausky.com
 * @date 2025/2/21
 */
@Slf4j
@Service
public class OaApprovalGeneration {

    @Autowired
    private OaFlowMapper oaFlowMapper;

    @Autowired
    private OaNodeMapper oaNodeMapper;

    @Autowired
    private SysDeptMapper sysDeptMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private OaSendMessageCenter oaSendMessageCenter;

    /**
     * @description: 新增流程与节点信息
     * @author: fyc
     * @email yuchuan.fu@chinausky.com
     * @date: 2025/2/21 19:23
     * @param: [oaFormDefinition]
     * @return: void
     **/
    @Async
    public void addGenerate(OaFormDefinition oaFormDefinition) {
        // 新增流程数据
        OaFlow oaFlow = new OaFlow();
        oaFlow.setFlowName(oaFormDefinition.getFormName() + "的流程");
        oaFlow.setDocType(oaFormDefinition.getFormSign());
        oaFlow.setCreateBy(oaFormDefinition.getCreateBy());
        oaFlow.setCreateTime(oaFormDefinition.getCreateTime());
        oaFlow.setDeptId(oaFormDefinition.getDeptId());
        oaFlow.setTenantId(oaFormDefinition.getTenantId());
        oaFlowMapper.insert(oaFlow);

        String flowInfo = oaFormDefinition.getFlowInfo();
        Integer flowId = oaFlow.getId();

        // 获取JSON字符串内的node节点信息
        if (StringUtils.isBlank(flowInfo)) {
            throw new BusinessException("节点信息为空！请联系管理员");
        }
        JSONObject jsonObject = JSONObject.parseObject(flowInfo);
        if (jsonObject.containsKey("node")) {
            List<JSONObject> nodeList = jsonObject.getJSONArray("node").toJavaList(JSONObject.class);
            // 用于记录上一个节点的 ID
            Integer lastNodeId = null;

            for (JSONObject node : nodeList) {
                String nodeNo = node.getString("nodebh");
                String nodeName = node.getString("nodeName");
                Integer nodeType = node.getInteger("nodeType");
                Integer nodeScope = node.getInteger("nodeScope");
                String proposer = null;
                Integer deptId2 = null;
                Integer jobId = null;

                switch (nodeScope) {
                    case 0:
                    case 2:
                        proposer = node.getString("appointApprover");
                        break;
                    case 1:
                        Long deptLeader = getDeptLeader();
                        if (deptLeader != null) {
                            proposer = deptLeader.toString();
                        }
                        break;
                    case 3:
                        deptId2 = node.getInteger("deptId");
                        break;
                    case 4:
                        jobId = node.getInteger("jobId");
                        break;
                    default:
                        throw new BusinessException("节点权限范围不正确");
                }

                OaNode oaNode = new OaNode();
                oaNode.setFlowId(flowId);
                oaNode.setNodeNo(nodeNo);
                oaNode.setNodeName(nodeName);
                oaNode.setNodeType(nodeType);
                oaNode.setNodeScope(nodeScope);
                oaNode.setProposer(proposer);
                oaNode.setDeptId(deptId2);
                oaNode.setJobId(jobId);
                // 使用上一个节点的 ID 作为父节点，第一个节点的父节点 ID 设置为 0
                oaNode.setParentId(lastNodeId == null ? 0 : lastNodeId);
                oaNodeMapper.insert(oaNode);

                // 更新 lastNodeId 为当前节点的 ID
                lastNodeId = oaNode.getId();
            }
        }
    }

    @Async
    public void updateGenerate(OaFormDefinition oaFormDefinition) {
        String flowInfo = oaFormDefinition.getFlowInfo();
        JSONObject jsonObject = JSONObject.parseObject(flowInfo);
        if (jsonObject.containsKey("node")) {
            List<JSONObject> nodeList = jsonObject.getJSONArray("node").toJavaList(JSONObject.class);
            // 用于记录上一个节点的 ID
            Integer lastNodeId = null;

            for (JSONObject node : nodeList) {
                String nodeNo = node.getString("nodebh");
                String nodeName = node.getString("nodeName");
                Integer nodeType = node.getInteger("nodeType");
                Integer nodeScope = node.getInteger("nodeScope");
                String proposer = null;
                Integer deptId2 = null;
                Integer jobId = null;

                switch (nodeScope) {
                    case 0:
                    case 1:
                    case 2:
                        proposer = node.getString("appointApprover");
                        break;
                    case 3:
                        deptId2 = node.getInteger("deptId");
                        break;
                    case 4:
                        jobId = node.getInteger("jobId");
                        break;
                    default:
                        throw new BusinessException("节点权限范围不正确");
                }

                OaNode oaNode = new OaNode();
                oaNode.setNodeName(nodeName);
                oaNode.setNodeType(nodeType);
                oaNode.setNodeScope(nodeScope);
                oaNode.setProposer(proposer);
                oaNode.setDeptId(deptId2);
                oaNode.setJobId(jobId);
                oaNode.setParentId(lastNodeId);
                oaNodeMapper.update(oaNode, new LambdaQueryWrapper<OaNode>().eq(OaNode::getNodeNo, nodeNo));
                lastNodeId = oaNode.getId();
            }
        }
    }

    private Long getDeptLeader() {
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        Integer tenantId = SecurityUtils.getTenantId();

        LambdaQueryWrapper<SysDept> deptLeaderQuery = new LambdaQueryWrapper<>();
        deptLeaderQuery.eq(SysDept::getTenantId, tenantId)
                .eq(SysDept::getDeptId, deptId)
                .select(SysDept::getLeader);
        SysDept deptLeader = sysDeptMapper.selectOne(deptLeaderQuery);
        String leader = deptLeader.getLeader();

        LambdaQueryWrapper<SysUser> userQuery = new LambdaQueryWrapper<>();
        userQuery.select(SysUser::getUserId)
                .eq(SysUser::getTenantId, tenantId)
                .eq(SysUser::getUserName, leader);
        SysUser sysUser = sysUserMapper.selectOne(userQuery);
        return sysUser.getUserId();
    }

    @Async
    public void generateApproval(OaApproval approval) {
        LambdaQueryWrapper<OaApproval> approvalQuery = new LambdaQueryWrapper<>();

    }

    // 根据单据类型获取流程信息
    public OaFlow selectBySign(String sign) {
        return oaFlowMapper.selectOne(Wrappers.lambdaQuery(OaFlow.class).eq(OaFlow::getDocType, sign));
    }

    // 根据流程ID获取节点信息
    public OaNode selectByFlowId(Integer flowId) {
        LambdaQueryWrapper<OaNode> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaNode::getFlowId, flowId)
                .orderByAsc(OaNode::getId);
        return oaNodeMapper.selectList(queryWrapper).get(0);
    }

}
